
/**
 * svnhook.p -
 *
 * (c) Copyright 2009 ABC Alon Blich Consulting Tech, Ltd.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Contact information
 *  Email: alonblich@gmail.com
 *  Phone: +972-54-218-8086
 */

{libos.i}

&if "{&opsys}" begins "win" &then

    {libwin.i}

&else

    {libunix.i}

&endif



define stream stIn.
define stream stOut.

define temp-table ttBuffer no-undo

    field iBufferId as int
    field cFullPath as char
    field cMsg      as char
    field lKeepMsg  as log

    index iBufferId is primary unique 
          iBufferId.



function getWorkingCopyStatus returns char private

    ( pcFullPath as char, plShowUpdates as log ) forward.



procedure svn_preopenFile:

    define input param pcFileName as char no-undo.

    define var cStatus  as char no-undo.
    define var lUpdate  as log no-undo.

    pcFileName = os_getFullPath( pcFileName ).
    if pcFileName = ? then return.



    cStatus = getWorkingCopyStatus( pcFileName, yes ).

    if cStatus <> ? and substr( cStatus, 8, 1 ) = "*" then do:

        run confirmUpdate( pcFileName, output lUpdate ).

        if lUpdate then
            run updateWorkingCopy( pcFileName, output cStatus ).

        /*** the commit has to be verified afterwards */

    end. /* cStatus <> ? */

end procedure. /* svn_preopenFile */

procedure svn_openFile:

    define input param piBufferId as int no-undo.
    define input param pcFileName as char no-undo.

    define buffer ttBuffer for ttBuffer.

    define var cStatus as char no-undo.

    pcFileName = os_getFullPath( pcFileName ).
    if pcFileName = ? then return.



    find first ttBuffer
         where ttBuffer.iBufferId = piBufferId
         use-index iBufferId
         no-lock no-error.

    if not avail ttBuffer then do:

        create ttBuffer.
        assign
            ttBuffer.iBufferId  = piBufferId
            ttBuffer.cFullPath  = pcFileName
            ttBuffer.cMsg       = ""
            ttBuffer.lKeepMsg   = ?.

    end. /* not avail */

end procedure. /* svn_openFile */

procedure svn_closeFile:

    define input param piBufferId as int no-undo.

    define buffer ttBuffer for ttBuffer.

    find first ttBuffer
         where ttBuffer.iBufferId = piBufferId
         use-index iBufferId
         no-lock no-error.

    if avail ttBuffer then delete ttBuffer.

end procedure. /* svn_closeFile */

procedure svn_saveFile:

    define input param piBufferId as int no-undo.
    define input param pcFileName as char no-undo.

    define buffer ttBuffer for ttBuffer.

    define var cFileName    as char no-undo.
    define var cStatus      as char no-undo.
    define var cStatus2     as char no-undo.
    
    pcFileName = os_getFullPath( pcFileName ).
    if pcFileName = ? then return.

    find first ttBuffer
         where ttBuffer.iBufferId = piBufferId
         use-index iBufferId
         no-lock no-error.

    if not avail ttBuffer then do: /* new file not opened file and saved for the first time */

        create ttBuffer.
        assign
            ttBuffer.iBufferId  = piBufferId
            ttBuffer.cFullPath  = pcFileName
            ttBuffer.cMsg       = ""
            ttBuffer.lKeepMsg   = ?.

    end. /* not avail */

    if ttBuffer.cFullPath <> pcFileName then do:

        assign
            cStatus     = getWorkingCopyStatus( ttBuffer.cFullPath, no )
            cStatus2    = getWorkingCopyStatus( pcFileName, no ).

        if cStatus <> ? and cStatus2 <> ? then do:

            if ttBuffer.lKeepMsg = no
            or ttBuffer.lKeepMsg = ? then
                run updateMsg( input-output ttBuffer.cMsg, input-output ttBuffer.lKeepMsg ).

            cFileName = os_getTempFile( "", ".dat" ).
            os-copy value( pcFileName ) value( cFileName ).

            if cStatus2 <> "?" then
                run deleteWorkingCopy( pcFileName ).
            else os-delete value( pcFileName ).

            if cStatus <> "?" then
                run copyWorkingCopy( ttBuffer.cFullPath, pcFileName ).

            os-copy value( cFileName ) value( pcFileName ).
            os-delete value( cFileName ).

            run commitWorkingCopy( pcFileName, ttBuffer.cMsg ).

        end. /* cStatus and cStatus2 <> ? */

        else
        if cStatus2 <> ? then do:

            if ttBuffer.lKeepMsg = no 
            or ttBuffer.lKeepMsg = ? then
                run updateMsg( input-output ttBuffer.cMsg, input-output ttBuffer.lKeepMsg ).

            if cStatus2 = "?" then
                run addWorkingCopy( pcFileName ).

            run commitWorkingCopy( pcFileName, ttBuffer.cMsg ).

        end. /* cStatus = ? */

        ttBuffer.cFullPath = pcFileName.

    end. /* else */

    else do:
        
        cStatus = getWorkingCopyStatus( pcFileName, no ).

        if cStatus <> ? and cStatus <> "" then do:
        
            if ttBuffer.lKeepMsg = no
            or ttBuffer.lKeepMsg = ? then
                run updateMsg( input-output ttBuffer.cMsg, input-output ttBuffer.lKeepMsg ).

            if cStatus = "?" then
                run addWorkingCopy( ttBuffer.cFullPath ).
            
            run commitWorkingCopy( ttBuffer.cFullPath, ttBuffer.cMsg ).
            
        end. /* cStatus <> ? */

    end. /* cFullPath = pcFileName */

    /*** the commit has to be verified afterwards */

end procedure. /* svn_saveFile */



procedure updateMsg private:

    define input-output param pcMsg     as char no-undo.
    define input-output param plKeepMsg as log no-undo.

    &if "{&window-system}" = "tty" &then
    
        form
            space(1)
            "Message"
            skip
        
            space(1)
            pcMsg       view-as editor scrollbar-vertical size 50 by 10 no-label
            space(1)
        
            skip
            space(1)
            plKeepMsg   view-as toggle-box label "Keep message"
        with frame frmMsg title "Subversion" centered view-as dialog-box.

        assign
            frame frmMsg:parent     = current-window
            frame frmMsg:column     = ( current-window:width-chars - frame frmMsg:width-chars ) / 2 + 1
            frame frmMsg:row        = ( current-window:height-chars - frame frmMsg:height-chars ) / 2 + 1.

    &else

        form
            skip(.5)
            space(3) 
            "Message"
            skip

            space(3)
            pcMsg       no-label view-as editor size 50 by 10 max-chars 100
            space(3)

            skip(.75)
            space(3)
            plKeepMsg   label "Keep Message" view-as toggle-box
            skip(.5)
        with frame frmMsg font 5 three-d title "Subversion" view-as dialog-box.

        assign
            frame frmMsg:parent     = current-window
            frame frmMsg:x          = ( session:work-area-width-pixels  - frame frmMsg:width-pixels ) / 2 + 1   - current-window:x
            frame frmMsg:y          = ( session:work-area-height-pixels - frame frmMsg:height-pixels ) / 2  + 1 - current-window:y.

    &endif

    on "end-error" of frame frmMsg anywhere do:

        return no-apply.

    end. /* end-error */



    if plKeepMsg = ? then
       plKeepMsg = yes. /* default */

    update 
        pcMsg
        plKeepMsg
            go-on( "return" )

    with frame frmMsg.

    /*** special character have to be escaped */

end procedure. /* updateMsg */

procedure confirmUpdate private:

    define input    param pcFullPath    as char no-undo.
    define output   param plUpdate      as log no-undo.

    plUpdate = no.

    message 
        'A newer revision of "' + pcFullPath + '" exists on the server.'
        skip(1)
        'Update working copy ?'
    view-as alert-box buttons yes-no title "Subversion" update plUpdate.

end procedure. /* confirmUpdate */



function getWorkingCopyStatus returns char private ( 

    pcFullPath      as char, 
    plShowUpdates   as log ): /* forces svn to check if there's a more up-to-date version in the repository.
                                 this options causes a long delay and is only required when opening a file
                                 to check if the version in the working copy is out-of-date. */

    define var cFile    as char no-undo.
    define var cDir     as char no-undo.
    define var cExt     as char no-undo.

    define var retval   as char no-undo.
    define var str      as char no-undo.
    
    run os_breakPath(
        input   pcFullPath,
        output  cDir,
        output  cFile,
        output  cExt ).

    if not os_isDirExists( cDir + "/.svn" ) then return ?.

    cFile = os_getTempFile( "", ".out" ).

    &if "{&opsys}" begins "win" &then

        run win_batch(
            input 'svn status "' + pcFullPath  + '" -v' + ( if plShowUpdates then ' -u' else '' ) + ' > "' + cFile + '"',
            input 'silent,wait' ).

    &else
        
        run unix_bash(
            input 'svn status "' + pcFullPath + '" -v' + ( if plShowUpdates then ' -u' else '' ) + ' > "' + cFile + '"',
            input 'silent,wait' ).

    &endif

    retval = ?.

    if search( cFile ) <> ? then do:

        str = "".
        
        input stream stIn from value( cFile ).
        
        do on endkey undo, leave:
            import stream stIn unformatted str.
        end.

        input stream stIn close.
        
        if str = "" then
             retval = ?.
        else retval = substr( str, 1, 8 ).

    end. /* search <> ? */
    
    os-delete value( cFile ).
    
    return retval.

end function. /* getWorkingCopyStatus */

procedure addWorkingCopy private:

    define input param pcFullPath as char no-undo.

    &if "{&opsys}" begins "win" &then

        run win_batch(
            input 'svn add "' + pcFullPath + '"',
            input 'silent,wait' ).

    &else

        run unix_bash(
            input 'svn add "' + pcFullPath + '"',
            input 'silent,wait' ).

    &endif

end procedure. /* addWorkingCopy */

procedure commitWorkingCopy private:

    define input param pcFullPath   as char no-undo.
    define input param pcMsg        as char no-undo.

    &if "{&opsys}" begins "win" &then

        run win_batch(
            input 'svn commit "' + pcFullPath + '" -m "' + replace( pcMsg, '"', '' ) + '"',
            input 'silent,wait' ).

    &else

        run unix_bash(
            input 'svn commit "' + pcFullPath + '" -m "' + replace( pcMsg, '"', '' ) + '"',
            input 'silent,wait' ).

    &endif

end procedure. /* commitWorkingCopy */

procedure copyWorkingCopy private:

    define input param pcSource as char no-undo.
    define input param pcTarget as char no-undo.

    &if "{&opsys}" begins "win" &then

        run win_batch(
            input 'svn copy "' + pcSource + '" "' + pcTarget + '"',
            input 'silent,wait' ).

    &else

        run unix_bash(
            input 'svn copy "' + pcSource + '" "' + pcTarget + '"',
            input 'silent,wait' ).

    &endif

end procedure. /* copyWorkingCopy */

procedure deleteWorkingCopy private:

    define input param pcFullPath as char no-undo.

    &if "{&opsys}" begins "win" &then

        run win_batch(
            input 'svn delete "' + pcFullPath + '" --force',
            input 'silent,wait' ).

    &else

        run unix_bash(
            input 'svn delete "' + pcFullPath + '" --force',
            input 'silent,wait' ).

    &endif

end procedure. /* deleteWorkingCopy */

procedure updateWorkingCopy private:

    define input    param pcFullPath    as char no-undo.
    define output   param pcStatus      as char no-undo.

    define var cFileName    as char no-undo.
    define var str          as char no-undo.

    cFileName = os_getTempFile( "", ".out" ).

    &if "{&opsys}" begins "win" &then

        run win_batch(
            input 'svn update "' + pcFullPath + '" > "' + cFileName + '"',
            input 'silent,wait' ).

    &else

        run unix_bash(
            input 'svn update "' + pcFullPath + '" > "' + cFileName + '"',
            input 'silent,wait' ).

    &endif

    pcStatus = ?.

    if search( cFileName ) <> ? then do:

        str = "".

        input stream stIn from value( cFileName ).

        do on endkey undo, leave:
            import stream stIn unformatted str.
        end.

        input stream stIn close.

        if str = "" then
             pcStatus = ?.
        else pcStatus = substr( str, 1, 1 ).

    end. /* search <> ? */

    os-delete value( cFileName ).

end procedure. /* updateWorkingCopy */
